use std::sync::Arc;

use clap::Parser;
use futures::future::join_all;

use base::RemoteConfig;
use tcp_relay_client::relay_client::relay_client;

/// TCP relay client
#[derive(Parser, Debug, Clone)]
#[clap(author, version, about, long_about = None)]
pub struct Args {
    /// Access token
    #[clap(short, long)]
    pub token: String,

    /// Relay server
    #[clap(short, long, default_value = "http://127.0.0.1:8000")]
    pub relay_url: String,

    /// Listen address
    #[clap(short, long, default_value = "127.0.0.1")]
    pub listen_address: String,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    let args: Args = Args::parse();
    let args = Arc::new(args);

    // Get server relay configuration (fetch the list of port to forward)
    let url = format!("{}/config", args.relay_url);
    log::info!("Retrieving configuration on {}", url);
    let req = reqwest::Client::new().get(url)
        .header("Authorization", format!("Bearer {}", args.token))
        .send()
        .await?;
    if req.status().as_u16() != 200 {
        log::error!("Could not retrieve configuration! (got status {})", req.status());
        std::process::exit(2);
    }
    let conf = req.json::<RemoteConfig>()
        .await?;

    // Start to listen port
    let mut handles = vec![];
    for port in conf {
        let listen_address = format!("{}:{}", args.listen_address, port.port);

        let h = tokio::spawn(relay_client(
            args.token.clone(),
            port.id,
            args.listen_address.clone(),
            listen_address,
        ));
        handles.push(h);
    }

    join_all(handles).await;

    Ok(())
}