use std::sync::Arc;

use actix_web::{App, Error, HttpResponse, HttpServer, middleware, Responder, web};
use actix_web::dev::ServiceRequest;
use actix_web::error::ErrorUnauthorized;
use actix_web::web::Data;
use actix_web_httpauth::extractors::bearer::BearerAuth;
use actix_web_httpauth::middleware::HttpAuthentication;
use clap::Parser;

use base::RelayedPort;
use tcp_relay_server::args::Args;

async fn auth_validator(
    req: ServiceRequest,
    creds: BearerAuth,
) -> Result<ServiceRequest, (Error, ServiceRequest)> {
    let args: &Data<Arc<Args>> = req.app_data().unwrap();
    if args.tokens.iter().any(|t| t == creds.token()) {
        Ok(req)
    } else {
        Err((ErrorUnauthorized("invalid token"), req))
    }
}

pub async fn hello_route() -> &'static str {
    "TCP relay. Hello world!"
}

pub async fn config_route(data: Data<Arc<Args>>) -> impl Responder {
    HttpResponse::Ok().json(
        data.ports.iter()
            .enumerate()
            .map(|(id, port)| RelayedPort { id, port: port + data.increment_ports })
            .collect::<Vec<_>>()
    )
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    let args: Args = Args::parse();
    let args = Arc::new(args);

    if args.ports.is_empty() {
        log::error!("No port to forward!");
        std::process::exit(2);
    }

    if args.tokens.is_empty() {
        log::error!("No tokens specified!");
        std::process::exit(3);
    }

    log::info!("Starting relay on http://{}", args.listen_address);

    let args_clone = args.clone();
    HttpServer::new(move || {
        App::new()
            .wrap(middleware::Logger::default())
            .wrap(HttpAuthentication::bearer(auth_validator))
            .app_data(Data::new(args_clone.clone()))
            .route("/", web::get().to(hello_route))
            .route("/config", web::get().to(config_route))
    })
        .bind(&args.listen_address)?
        .run()
        .await
}